/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: WsdlUtil.java,v 1.6 2007/02/27 22:31:04 sshirasu Exp $
 */

package xsul5.wsdl;

import java.net.URI;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import org.xmlpull.infoset.XmlAttribute;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.XmlNamespace;
import xsul5.MLogger;
import xsul5.XmlConstants;
import xsul5.wsdl.soap.WsdlSoapAddress;
import xsul5.wsdl.soap.WsdlSoapBinding;
import xsul5.wsdl.soap.WsdlSoapOperation;
import xsul5.wsdl.soap.WsdlSoapUtil;



/**
 * WSLD 1.1 constants, functions, etc
 *
 * @version $Revision: 1.6 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class WsdlUtil {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public final static String WSDL_TRANSPORT_SOAP =
    "http://schemas.xmlsoap.org/soap/http";
    public final static XmlNamespace WSDL_SOAP_NS = builder.newNamespace(
        "http://schemas.xmlsoap.org/wsdl/soap/");
    public final static XmlNamespace WSDL_SOAP12_NS = builder.newNamespace(
        "http://schemas.xmlsoap.org/wsdl/soap12/");
    public final static String FEATURE_EL = "feature";
    public final static String URI_ATTR = "uri";
    public static String REQUIRED_ATTR = "required";
    private final static String LOCATION = "location";
    
    public static void replaceWsdlSoapAddr(WsdlPort wsdlPort, String serviceLoc) throws WsdlException {
        XmlElement soapAddr = getWsdlSoapAddress(wsdlPort);
        if (soapAddr == null) {
            throw new WsdlException("could not find wsdl/service/port/address in " + wsdlPort);
        }
        XmlAttribute locationAttr = soapAddr.attribute(null, LOCATION);
        if (locationAttr != null) {
            soapAddr.removeAttribute(locationAttr);
        }
        soapAddr.setAttributeValue(LOCATION, serviceLoc);
    }
    
    private static XmlElement getWsdlSoapAddress(WsdlPort wsdlPort) throws WsdlException {
        XmlElement soapAddr = wsdlPort.xml().element(WSDL_SOAP_NS, WsdlSoapAddress.TYPE_NAME);
        if (soapAddr == null) {
            soapAddr = wsdlPort.xml().element(WSDL_SOAP12_NS, WsdlSoapAddress.TYPE_NAME);
            if (soapAddr == null) {
                return null;
            }
        }
        return soapAddr;
    }
    
    public static URI getWsdlSoapAddress(WsdlDefinitions def) throws WsdlException {
        for (WsdlService oneService : def.services()) {
            for (WsdlPort port : oneService.ports()) {
                XmlElement wsdlSoapAddress = getWsdlSoapAddress(port);
                if (wsdlSoapAddress != null) {
                    String location = wsdlSoapAddress.attributeValue(LOCATION);
                    return URI.create(location);
                }
            }
        }
        return null;
    }
    
    public static void replaceWsdlSoapAddr(WsdlDefinitions def, String serviceLoc) throws WsdlException {
        for (WsdlService oneService : def.services()) {
            for (WsdlPort port : oneService.ports()) {
                WsdlUtil.replaceWsdlSoapAddr(port, serviceLoc);
            }
        }
        
    }
    
    public static boolean hasSoapDocLiteralPort(WsdlDefinitions def) throws WsdlException {
        for (WsdlService oneService : def.services()) {
            for (WsdlPort port : oneService.ports()) {
                if (getWsdlSoapAddress(port) != null) {
                    return true;
                }
            }
        }
        return false;
    }
    
    public static void removeAllBindingsAndServices(WsdlDefinitions definitions) throws WsdlException {
        List<XmlElement> toRemove = new LinkedList<XmlElement>();
        for (WsdlBinding binding : definitions.bindings()) {
            toRemove.add(binding.xml());
        }
        for (WsdlService service : definitions.services()) {
            toRemove.add(service.xml());
        }
        for (XmlElement el : toRemove) {
            definitions.xml().removeElement(el);
        }
    }
    
    public static WsdlDefinitions createCWSDL(WsdlDefinitions definitions,
                                              URI uri) throws WsdlException {
        // Create a new List to avoid ConcurrentModificationException.
        List<WsdlPortType> portTypes = new ArrayList<WsdlPortType>();
        for (WsdlPortType portType : definitions.portTypes()) {
            portTypes.add(portType);
        }

        for (WsdlPortType portType : portTypes) {
            createCWSDL(definitions, portType, uri);
        }
        return definitions;
    }
    
    public static void createCWSDL(WsdlDefinitions definitions,
                                   WsdlPortType portType, URI uri) {
        
        final XmlNamespace NS = definitions.xml().getNamespace();
        
        // <wsdl:binding>
        String bindingName = portType.getName() + "SoapBinding";
        WsdlSoapBinding binding = new WsdlSoapBinding(bindingName, portType
                                                      .getQName());
        binding.setStyle(WsdlSoapBinding.STYLE.document);
        binding.setTransport(WsdlSoapBinding.HTTP_TRANSPORT);
        definitions.addBinding(binding);
        
        for (WsdlPortTypeOperation portTypeOperation : portType.operations()) {
            // <wsdl:operation>
            String operationName = portTypeOperation.getName();
            WsdlSoapOperation operation = new WsdlSoapOperation(operationName);
            // useful for WS-Addressing
            operation.setSoapAction(URI.create(definitions.getTargetNamespace()
                                               + operationName));
            operation.setStyle(WsdlSoapBinding.STYLE.document);
            binding.xml().addElement(operation.xml());
            binding.setStyle(WsdlSoapBinding.STYLE.document);
            // <wsdl:input>
            XmlElement input = operation.xml().addElement(NS, "input");
            XmlElement inputBody = input.addElement(WsdlSoapUtil.WSDL_SOAP_NS,
                                                    "body");
            inputBody.setAttributeValue("use", "literal");
            // <wsdl:output>
            XmlElement output = operation.xml().addElement(NS, "output");
            XmlElement outputBody = output.addElement(
                WsdlSoapUtil.WSDL_SOAP_NS, "body");
            outputBody.setAttributeValue("use", "literal");
        }
        
        // <wsdl:service>
        String serviceName = portType.getName() + "Service";
        WsdlService service = definitions.addService(serviceName);
        String portName = portType.getName() + "SoapPort";
        WsdlPort port = service.addPort(portName, binding.getQName());
        WsdlSoapAddress address = new WsdlSoapAddress(uri);
        port.xml().addElement(address.xml());
    }
    
}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/


